@@ -3,14 +3,14 @@
 #
 # Crypto-licensing -- Cryptographically signed licensing, w/ Cryptocurrency payments
 #
-# Copyright (c) 2022, Hard Consulting Corporation.
+# Copyright (c) 2022, Dominion Research & Development Corp.
 #
-# Cpppo is free software: you can redistribute it and/or modify it under the
-# terms of the GNU General Public License as published by the Free Software
-# Foundation, either version 3 of the License, or (at your option) any later
-# version.  See the LICENSE file at the top of the source tree.
+# Crypto-licensing is free software: you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the Free
+# Software Foundation, either version 3 of the License, or (at your option) any
+# later version.  See the LICENSE file at the top of the source tree.
 #
-# Cpppo is distributed in the hope that it will be useful, but WITHOUT ANY
+# It is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 # A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
@@ -398,8 +398,8 @@ CONFIG_FILE			= CONFIG_BASE+'.cfg'    # Default application configuration file
 
 
 class Timestamp( datetime.datetime ):
-    """A simple Timestamp that can be specified from a Timestamp or datetime.datetime, and is always local to a
-    specific timezone, and formats simply and deterministically.
+    """A simple Timestamp that can be specified from a Timestamp or datetime.datetime, and is always
+    local to a specific timezone, and formats simply and deterministically.
 
     """
     UTC				= pytz.UTC
@@ -417,7 +417,8 @@ class Timestamp( datetime.datetime ):
 
     def __new__( cls, *args, **kwds ):
         """Since datetime.datetime is immutable, we must use __new__ and return one."""
-        kargs			= dict( zip( ('year', 'month', 'day', 'hour', 'minute', 'second', 'microsecond', 'tzinfo' ), args ))
+        kargs			= dict( zip(
+            ('year', 'month', 'day', 'hour', 'minute', 'second', 'microsecond', 'tzinfo' ), args ))
         assert len( kargs ) == len( args ), \
             "Too many args provided"
         kdup			= set( kargs ).intersection( kwds )
@@ -450,15 +451,15 @@ class Timestamp( datetime.datetime ):
         return datetime.datetime.__new__( cls, **kwds )
 
     def render( self, tzinfo=None, ms=True, tzdetail=None ):
-        """Render the time in the specified zone (default: local), optionally with milliseconds (default:
-        True).
+        """Render the time in the specified zone (default: local), optionally with milliseconds
+        (default: True).
 
         If the resultant timezone is not UTC, include the timezone full name in the output by
         default.  Thus, the output is always deterministic (only UTC times may every be output
         without timezone information).
 
-        If tzdetail is supplied, if bool/str and Truthy, always include full timezone name;
-        if Falsey, include abbreviation (warning: NOT parsable, because abbreviations are
+        If tzdetail is supplied, if bool/str and Truthy, always include full timezone name; if
+        Falsey, include abbreviation (warning: NOT parsable, because abbreviations are
         non-deterministic!).  If int and Truthy, always include full numeric timezone offset; if
         Falsey; only with non-UTC timezones.
 
@@ -492,9 +493,10 @@ class Timestamp( datetime.datetime ):
         return int( self.timestamp() )
 
     def timestamp( self ):
-        """Exists in Python 3.3+.  Convert a timezone-aware datetime to a UNIX timestamp.  You'd think
-        strftime( "%s.%f" )?  You'd be wrong; a timezone-aware datetime should always strftime
-        to the same (correct) UNIX timestamp via its "%s" format, but this also doesn't work.
+        """Exists in Python 3.3+.  Convert a timezone-aware datetime to a UNIX timestamp.  You'd
+        think strftime( "%s.%f" )?  You'd be wrong; a timezone-aware datetime should always
+        strftime to the same (correct) UNIX timestamp via its "%s" format, but this also doesn't
+        work.
 
         Convert the time to a UTC time tuple, then use calendar.timegm to take a UTC time tuple and
         compute the UNIX timestamp.
@@ -507,14 +509,27 @@ class Timestamp( datetime.datetime ):
 
     # Comparisons.  Always equivalent to lexicographically, in UTC to 3 decimal places.  However,
     # we'll compare numerically, to avoid having to render/compare strings; if the <self>.value is
-    # within _epsilon (default: 0.001) of <rhs>.value, it is considered equal.
+    # within _epsilon (default: 0.001) of <rhs>.value, it is considered equal.  Pypy2
+    # re-implements the pytz library w/ some comparisons against raw datetime.datetimes, so we
+    # have to support them directly; they have no .timestamp().  However, pypy2 has other issues,
+    # so we don't support it -- it is an optional optimization for Python2 code, anyway...
     def __lt__( self, rhs ):
-        assert isinstance( rhs, Timestamp )
-        return self.timestamp() + self.__class__._epsilon < rhs.timestamp()
+        assert isinstance( rhs, (Timestamp, datetime.datetime) ), \
+            "Expected Timestamp/datetime, got: {!r}".format( rhs )
+        try:
+            rhs_ts		= rhs.timestamp()
+        except AttributeError:
+            rhs_ts		= calendar.timegm( rhs.utctimetuple() ) + rhs.microsecond / 1000000
+        return self.timestamp() + self.__class__._epsilon < rhs_ts
 
     def __gt__( self, rhs ):
-        assert isinstance( rhs, Timestamp )
-        return self.timestamp() - self.__class__._epsilon > rhs.timestamp()
+        assert isinstance( rhs, (Timestamp, datetime.datetime) ), \
+            "Expected Timestamp/datetime, got: {!r}".format( rhs )
+        try:
+            rhs_ts		= rhs.timestamp()
+        except AttributteError:
+            rhs_ts		= calendar.timegm( rhs.utctimetuple() ) + rhs.microsecond / 1000000
+        return self.timestamp() - self.__class__._epsilon > rhs_ts
 
     def __le__( self, rhs ):
         return not self.__gt__( rhs )
@@ -532,8 +547,10 @@ class Timestamp( datetime.datetime ):
     # datetime.datetime is immutable, so cannot implemente __i{add/sub}__.  Converts to timedelta,
     # and uses underlying __{add,sub}__.
     def __add__( self, rhs ):
-        """Convert any int/float/str, etc. to a Duration/timedelta, and add it to the current Timestamp,
-        retaining its timezone preference.
+        """
+
+        Convert any int/float/str, etc. to a Duration/timedelta, and add it to the current
+        Timestamp, retaining its timezone preference.
 
         """
         if not isinstance( rhs, Duration):
@@ -550,7 +567,7 @@ class Timestamp( datetime.datetime ):
 
         """
         if isinstance( rhs, datetime.datetime ):
-            # Subtracting datetimes; result is a Duration; obtain from existing __sub__ returning a timedelta
+            # Subtracting datetimes; result is a Duration; from existing __sub__ --> timedelta
             return Duration( super( Timestamp, self ).__sub__( rhs ))
         # Subtracting something else; must be convertible by Duration into a timedelta
         rhs		= Duration( rhs )
